@@ -24,13 +24,16 @@
 #include "config.h"
 
 #include <gstdio.h>
-#include <locale.h>
-#include <string.h>
-#include <unistd.h>
-#include <stdio.h>
-
 #include <gi18n.h>
 
+#include <string.h>
+#include <stdio.h>
+#include <locale.h>
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #include "gvdb/gvdb-builder.h"
 #include "strinfo.c"
 
@@ -1965,13 +1968,13 @@ main (int argc, char **argv)
 
       if (files->len == 0)
         {
-          fprintf (stderr, _("No schema files found: "));
+          fprintf (stdout, _("No schema files found: "));
 
           if (g_unlink (target))
-            fprintf (stderr, _("doing nothing.\n"));
+            fprintf (stdout, _("doing nothing.\n"));
 
           else
-            fprintf (stderr, _("removed existing output file.\n"));
+            fprintf (stdout, _("removed existing output file.\n"));
 
           return 0;
         }
