@@ -29,14 +29,13 @@
  */
 
 #include "config.h"
+#include "glibconfig.h"
 
 #define DEBUG_MSG(x)	/* */
 #ifdef G_ENABLE_DEBUG
 /* #define DEBUG_MSG(args)	g_message args ; */
 #endif
 
-#include "glib.h"
-
 #include <time.h>
 #include <string.h>
 #include <stdlib.h>
@@ -46,6 +45,18 @@
 #include <windows.h>
 #endif
 
+#include "gdate.h"
+
+#include "gconvert.h"
+#include "gmem.h"
+#include "gstrfuncs.h"
+#include "gtestutils.h"
+#include "gthread.h"
+#include "gunicode.h"
+
+#ifdef G_OS_WIN32
+#include "garray.h"
+#endif
 
 GDate*
 g_date_new (void)
@@ -945,6 +956,8 @@ g_date_set_time (GDate *date,
  * @tv_usec member is ignored, because #GDate can't make use of the
  * additional precision.
  *
+ * The time to date conversion is done using the user's current timezone.
+ *
  * Since: 2.10
  */
 void
